package info.jbcs.minecraft.randomite;

import java.util.ArrayList;
import java.util.Random;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.enchantment.EnchantmentHelper;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.stats.StatList;
import net.minecraft.world.World;

public class BlockRandomite extends BlockTexturedOre {
	Random	rand	= new Random();

	public BlockRandomite(int id, Material mat, Block base) {
		super(id, mat, base);
	}

	@Override
	public void harvestBlock(World world, EntityPlayer player, int x, int y, int z, int meta) {
		player.addStat(StatList.mineBlockStatArray[this.blockID], 1);
		player.addExhaustion(0.025F);

		if (canSilkHarvest(world, player, x, y, z, meta) && EnchantmentHelper.getSilkTouchModifier(player)) {
			ItemStack itemstack = createStackedBlock(meta);

			if (itemstack != null) {
				this.dropBlockAsItem_do(world, x, y, z, itemstack);
			}
		} else {
			int i1 = EnchantmentHelper.getFortuneModifier(player);
			this.dropBlockAsItem(world, x, y, z, meta, i1);
		}
	}

	ChunkEvaluation	lastEvaluation	= null;

	@Override
	public ArrayList<ItemStack> getBlockDropped(World world, int x, int y, int z, int metadata, int fortune) {
		ArrayList<ItemStack> ret = new ArrayList<ItemStack>();
		if (world.isRemote)
			return ret;

		double totalProbabiltiy = 0;
		double probability = 0;
		ItemStack result = null;
		
		

		ChunkEvaluation evaluation = ChunkEvaluator.evaluate(world, x / 16, z / 16);
		if (evaluation == null) {
			ret.add(new ItemStack(this, 1, 0));
			return ret;
		}

		if (lastEvaluation != evaluation)
			Randomite.debug("Chunk " + evaluation.key + " evaluation:");
		
		for (ChunkEvaluatorItemEntry entry : evaluation.itemMap[fortune].values()) {
			totalProbabiltiy += 1.0 / entry.count;
			
			if (lastEvaluation != evaluation)
				Randomite.debug("  "+Item.itemsList[entry.id].getUnlocalizedName()+"|"+entry.damage+"[a="+entry.average+", d="+entry.dispersion+"]: "+entry.count+"x");
		}
		
		lastEvaluation=evaluation;

		double value = world.rand.nextDouble() * totalProbabiltiy;
		for (ChunkEvaluatorItemEntry entry : evaluation.itemMap[fortune].values()) {
			probability += 1.0 / entry.count;
			if (probability < value)
				continue;

			int min = (int) Math.round(Randomite.countMultiplier * (entry.average - entry.dispersion));
			int max = (int) Math.round(Randomite.countMultiplier * (entry.average + entry.dispersion));

			int count = min + ((max == min) ? 0 : rand.nextInt(max - min));
			if (count < 1)
				count = 1;

			result = new ItemStack(entry.id, count, entry.damage);
			break;

		}

		if (result == null)
			result = new ItemStack(this, 1, 0);

		ret.add(result);
		return ret;
	}

	@Override
	public boolean canSilkHarvest(World world, EntityPlayer player, int x, int y, int z, int metadata) {
		return true;
	}

}
