package info.jbcs.minecraft.randomite;

import java.util.TreeSet;

import net.minecraft.block.Block;
import net.minecraft.entity.Entity;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.EnumSkyBlock;
import net.minecraft.world.World;
import net.minecraft.world.WorldSettings;
import net.minecraft.world.biome.BiomeGenBase;
import net.minecraft.world.chunk.Chunk;
import net.minecraft.world.chunk.IChunkProvider;
import cpw.mods.fml.common.registry.GameRegistry;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

public class FakeWorld extends World {
	World			realWorld;
	Chunk			chunk;
	int				cx;
	int				cz;
	int				x1;
	int				z1;
	int				x2;
	int				z2;
	static int		fortuneLevels	= 5;
	ChunkEvaluation	eval;
	
	public FakeWorld(World realWorld, int x, int z) {
		super(new FakeSaveHandler(new DreivedWorldInfoForDimension(realWorld.getWorldInfo(),realWorld.provider.dimensionId)), realWorld.getWorldInfo().getWorldName(), new WorldSettings(realWorld.getWorldInfo()), null, realWorld.theProfiler, realWorld.getWorldLogAgent());

		this.realWorld = realWorld;

		isRemote = false;

		chunk = chunkProvider.provideChunk(cx = x, cz = z);
		x1 = x * 16;
		z1 = z * 16;
		x2 = x * 16 + 16;
		z2 = z * 16 + 16;
	}

	void evaluateChunk(ChunkEvaluation eval) {
		this.eval = eval;
		chunkProvider.populate(chunkProvider, cx, cz);
		GameRegistry.generateWorld(cx, cz, this, chunkProvider, chunkProvider);
	}

	@Override
	protected IChunkProvider createChunkProvider() {
		IChunkProvider chunkProvider=provider.createChunkGenerator();

		return chunkProvider;
	}

	@Override
	public Entity getEntityByID(int i) {
		return null;
	}

	@Override
	public int getBlockId(int x, int y, int z) {
		if (x < x1 || x >= x2 || z < z1 || z >= z2)
			return 0;
		if (y < 0 || y >= 256)
			return 0;

		return chunk.getBlockID(x & 15, y, z & 15);
	}

	@Override
	public boolean setBlock(int x, int y, int z, int id, int meta, int flags) {
		if (x < x1 || x >= x2 || z < z1 || z >= z2)
			return false;
		if (y < 0 || y >= 256)
			return false;

		boolean res = chunk.setBlockIDWithMetadata(x & 15, y, z & 15, id, meta);

		Block block = Block.blocksList[id];
		if (block == null)
			return res;

		if (y > Randomite.maxTerrainLevel)
			return res;
		if (y < Randomite.minTerrainLevel)
			return res;

		if (Randomite.excludedBlocks[id])
			return res;

		TreeSet<ChunkEvaluatorItemEntry> entriesFromCurrentBlock = new TreeSet<ChunkEvaluatorItemEntry>();
		for (int fortuneLevel = 0; fortuneLevel < fortuneLevels; fortuneLevel++) {
			if(block instanceof BlockRandomite) continue;
			entriesFromCurrentBlock.clear();

			int count = block.quantityDropped(meta, fortuneLevel, rand);
			for (int i = 0; i < count; i++) {
				int idd = block.idDropped(meta, rand, 0);
				if (idd <= 0)
					continue;
				int dmg = block.damageDropped(meta);

				String key = idd + "|" + dmg;
				ChunkEvaluatorItemEntry entry = eval.itemMap[fortuneLevel].get(key);

				if (entry == null)
					entry = new ChunkEvaluatorItemEntry(idd, 1, dmg);
				else
					entry.count++;

				entriesFromCurrentBlock.add(entry);
				entry.currentBlockCount++;

				eval.itemMap[fortuneLevel].put(key, entry);
			}

			for (ChunkEvaluatorItemEntry entry : entriesFromCurrentBlock) {
				entry.blockCount++;
				double average = 1.0 * entry.count / entry.blockCount;
				entry.dispersion += (average - entry.currentBlockCount) * (average - entry.currentBlockCount);
				entry.currentBlockCount = 0;
			}
		}

		return res;
	}

	@Override
	public boolean setBlockMetadataWithNotify(int x, int y, int z, int meta, int flags) {
		if (x < x1 || x >= x2 || z < z1 || z >= z2)
			return false;
		if (y < 0 || y >= 256)
			return false;

		return chunk.setBlockMetadata(x & 15, y, z & 15, meta);
	}
	
    @Override
	public Chunk getChunkFromChunkCoords(int x, int z){
//    	if(x!=cx || z!=cz) return null;
    	
        return chunk;
    }


	@Override
	public TileEntity getBlockTileEntity(int i, int j, int k) {
		return null;
	}

	@Override
	@SideOnly(Side.CLIENT)
	public BiomeGenBase getBiomeGenForCoords(int i, int j) {

		return realWorld.getBiomeGenForCoords(i, j);
	}

	@Override
	public int getPrecipitationHeight(int par1, int par2) {
		return 0;
	}

	@Override
	public int getFullBlockLightValue(int par1, int par2, int par3) {
		return 0;
	}

	@Override
	public boolean canBlockSeeTheSky(int par1, int par2, int par3) {
		return false;
	}

	@Override
	public void updateLightByType(EnumSkyBlock par1EnumSkyBlock, int par2, int par3, int par4) {

	}
}
