package info.jbcs.minecraft.randomite;

import java.util.HashMap;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraftforge.common.Configuration;
import cpw.mods.fml.common.Mod;
import cpw.mods.fml.common.Mod.Init;
import cpw.mods.fml.common.Mod.Instance;
import cpw.mods.fml.common.Mod.PostInit;
import cpw.mods.fml.common.Mod.PreInit;
import cpw.mods.fml.common.SidedProxy;
import cpw.mods.fml.common.event.FMLInitializationEvent;
import cpw.mods.fml.common.event.FMLPostInitializationEvent;
import cpw.mods.fml.common.event.FMLPreInitializationEvent;
import cpw.mods.fml.common.network.NetworkMod;
import cpw.mods.fml.common.registry.GameRegistry;
import cpw.mods.fml.common.registry.LanguageRegistry;

@Mod(modid="Randomite", name="Randomite", version="1.0.0")
@NetworkMod(clientSideRequired=true)
public class Randomite {
	public static BlockTexturedOre blockRandomite;

	static Configuration config;

	@Instance("Randomite")
	public static Randomite instance;

	@SidedProxy(clientSide = "info.jbcs.minecraft.randomite.ProxyClient", serverSide = "info.jbcs.minecraft.randomite.Proxy")
	public static Proxy proxy;

	@PreInit
	public void preInit(FMLPreInitializationEvent event) {
		config = new Configuration(event.getSuggestedConfigurationFile());
		config.load();
		
		proxy.preInit();
	}
	
	static double countMultiplier;
	static boolean debugOutput;
	static int maxTerrainLevel;
	static int minTerrainLevel;
	static String excludedBlocksList="";
	static boolean[] excludedBlocks=new boolean[Block.blocksList.length];
	
	static void debug(String s){
		if(debugOutput) System.out.println("Randomite: "+s);
	}
	
	int getBlockId(String name,int id){
		return config.getBlock(name, id).getInt(id);
	}

	int getItemId(String name,int id){
		return config.getItem(name, id).getInt(id);
	}

	@Init
	public void init(FMLInitializationEvent event) {
		proxy.init();
		
		blockRandomite=(BlockRandomite) new BlockRandomite(getBlockId("randomite",2791),Material.rock,Block.stone).setUnlocalizedName("Randomite:randomiteOre").setHardness(1.5F).setResistance(10.0F);
		GameRegistry.registerBlock(blockRandomite,"randomiteOre");
		LanguageRegistry.addName(blockRandomite, "Randomite ore");
		blockRandomite.setCreativeTab(CreativeTabs.tabBlock);
		
		excludedBlocksList=config.get("general", "Randomite excludede blocks", "dirt, sand, gravel", "Comma separated list of block unlocalized names; those blocks will not affect what items randomite ore drops.").getString();
		
		
		debugOutput=config.get("general", "Debug output", false, "Print lots of debug information to STDOUT").getBoolean(false);
		countMultiplier=config.get("general", "Randomite count multiplier", 1.0, "Multiply the amount of items dropped from randomite ore by this value").getDouble(1.0);
		minTerrainLevel=config.get("general", "Randomite bottom terrain level", 0, "Randomite only drops items harvested from blocks above this level").getInt(0);
		maxTerrainLevel=config.get("general", "Randomite top terrain level", 50, "Randomite only drops items harvested from blocks below this level").getInt(50);
		
        GameRegistry.registerWorldGenerator(new RandomiteWorldGenerator(blockRandomite.blockID, 3, config.get("general", "Worldgen randomite ore amount", 65, "Amount of randomite to generate in the world; use 0 for none").getInt(65)));
	
        config.save();
	}

	@PostInit
	public void postInit(FMLPostInitializationEvent event) {
		HashMap<String,Block> blockMapping=new HashMap<String,Block>();
		for(Block block: Block.blocksList){
			if(block==null) continue;
			String name=block.getUnlocalizedName();
			if(name.startsWith("tile.")) name=name.substring(5);
			
			blockMapping.put(name.toLowerCase(), block);
		}
		
		for(String s: excludedBlocksList.split(",")){
			s=s.trim();
			if(s.isEmpty()) continue;
			
			Block block=blockMapping.get(s.toLowerCase());
			if(block==null){
				System.out.println("Bad block name in config: "+s);
				continue;
			}
			
			excludedBlocks[block.blockID]=true;
			debug("Excluding block "+block+"("+block.blockID+","+s+") from randomite blocks");
		}
	}
}



